# import pytest

# from click.testing import CliRunner

# from hpcflow import __version__
# from hpcflow.cli.cli import cli


# def test_version():
#     runner = CliRunner()
#     result = runner.invoke(cli, args="--version")
#     assert result.output.strip() == f"hpcflow, version {__version__}"
